<?php
namespace Teknisa\PLA\Modules\Parametros\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;


class Pla06700RegrasParaRevisaoDeReceita extends \Teknisa\GER\Controllers\TeknisaCrud {
	protected $parameterBagColumns;
	protected $crudService;
	protected $dataSourceManager;
	protected $environment;
	protected $parameterBag;
	protected $validQtdLiquida = "Qtde. Líquida do produto origem deve ser diferente da Qtde. Líquida do produto destino.";
	protected $validQtdBruta = "Qtde. Bruta do produto origem deve ser diferente da Qtde. Bruta do produto destino.";
	protected $validProdOrig = "Produto origem já cadastrado em produto destino.";
	protected $prodOrigInativo = "Regra não pode conter produto de origem inativo.";
	protected $prodDestInativo = "Regra não pode conter produto de destino inativo.";
	protected $util;
	protected $entityManager;

	public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, $entityManager, $util){
		$this->crudService 	        = $crudService;
	    $this->parameterBag         = $parameterBag;
		$this->dataSourceManager    = $dataSourceManager;
		$this->environment 			= $environment;
		$this->entityManager        = $entityManager;
		$this->util             	= $util;


		parent::__construct($crudService, $parameterBag);
	}
	
	public function getIdqtliqubrutToGrid(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $sql = "SELECT NVL(IDQTLIQUBRUT, 'L') AS IDQTLIQUBRUT FROM PARAMPLANEJA WHERE CDFILIAL = :CDFILIAL";
		$params = array(
            'CDFILIAL' => $row['CDFILIAL'],
		);
		$query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        $response->addDataSet(new DataSet("pla06700_getIdqtliqubrutToGrid", $result));
    }
    
    public function saveRegra(DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $oldRows = self::getRegras($rows[0]['CDFILIAL']);
            $rows = self::replaceDot($rows);
            self::validQtdRegra($rows, $oldRows);
            self::validProdInativo($rows);
            $dataset = new DataSet('pla06700_saveRegra', $rows);
            $dataset = new DTO\Request\DataSet($dataset, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            $this->dataSourceName = 'pla06700_regras_para_revisao_de_receita';
            self::save($dataset, $response);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function replaceDot($rows){
    	$rows[0]['QTPRODORIGRE'] = str_replace('.', '', $rows[0]['QTPRODORIGRE']);
    	$rows[0]['QTPRODORIGRE'] = str_replace(',', '.', $rows[0]['QTPRODORIGRE']);
    	if(!empty($rows[0]['QTPRODDESTRE'])){
	    	$rows[0]['QTPRODDESTRE'] = str_replace('.', '', $rows[0]['QTPRODDESTRE']);
	    	$rows[0]['QTPRODDESTRE'] = str_replace(',', '.', $rows[0]['QTPRODDESTRE']);
    	}
		return $rows;
    }
    
    public function validQtdRegra($rows, $oldRows){
    	if(!empty($rows[0]['CDPRODDESTRE'])){
            if($rows[0]['CDPRODORIGRE'] == $rows[0]['CDPRODDESTRE'] &&
			   $rows[0]['QTPRODORIGRE'] == $rows[0]['QTPRODDESTRE'] &&
			   $rows[0]['QTPRODORIGRE'] > 0){
				if($rows[0]['IDQTLIQUBRUT'] == 'B'){
					throw new \Exception($this->validQtdLiquida, 500);
				}else{
					throw new \Exception($this->validQtdBruta, 500);
				}   	
			}
			foreach($oldRows as $key => &$oldRow){
				if($rows[0]['CDPRODORIGRE'] == $oldRow['CDPRODDESTRE'] && 
				   $rows[0]['QTPRODORIGRE'] == $oldRow['QTPRODDESTRE'] &&
			       $rows[0]['QTPRODORIGRE'] > 0){
					throw new \Exception($this->validProdOrig, 500);
				}else if($rows[0]['CDPRODDESTRE'] == $oldRow['CDPRODORIGRE'] && 
					     $rows[0]['QTPRODDESTRE'] == $oldRow['QTPRODORIGRE']){
					throw new \Exception($this->validProdOrig, 500);
				}
			}
        }
    }
    
    public function validProdInativo($rows){
    	$newProdOrig = self::verificaProdutoInativo($rows, $rows[0]['CDARVORIG']);
		if($newProdOrig[0]['IDPRODATIVPR'] == 'N' || $newProdOrig[0]['IDPRODATIV'] == 'I'){
			throw new \Exception($this->prodOrigInativo, 500);
		}
		if(!empty($rows[0]['CDARVDEST'])){
			$newProdDest = self::verificaProdutoInativo($rows, $rows[0]['CDARVDEST']);
			if($newProdDest[0]['IDPRODATIVPR'] == 'N' || $newProdDest[0]['IDPRODATIV'] == 'I'){
				throw new \Exception($this->prodDestInativo, 500);
			}
		}
    }
    
    public function getRegras($cdfilial){
    	$sql = "SELECT R.NRORG, R.CDFILIAL, R.IDAPLICAREG, R.CDPRODORIGRE, ORIG.CDARVPROD AS CDARVORIG, ORIG.NMPRODUTO AS NMPRODORIG, R.QTPRODORIGRE, ORIG.SGUNIDADE AS UNIDORIG, R.CDPRODDESTRE, DEST.CDARVPROD AS CDARVDEST, DEST.NMPRODUTO AS NMPRODDEST, R.QTPRODDESTRE, DEST.SGUNIDADE AS UNIDDEST, DECODE(R.CDPRODDESTRE, '', 'E', 'S') AS STATUS FROM REGRAREVRECE R, PRODUTO ORIG,   PRODUTO DEST WHERE R.CDFILIAL = :CDFILIAL  AND R.CDPRODORIGRE = ORIG.CDPRODUTO AND R.CDPRODDESTRE = DEST.CDPRODUTO(+) ORDER BY ORIG.CDARVPROD";
		$params = array(
            'CDFILIAL' => $cdfilial,
		);
		$query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        return $result = $query->fetchAll();
    }
    
    public function verificaProdutoInativo($row, $prod){
    	$sql = "SELECT P.CDPRODUTO, P.IDPRODATIVPR, NVL(PF.IDPRODATIV, 'A') IDPRODATIV FROM PRODFILI PF, PRODUTO P WHERE (P.CDARVPROD = :CDARVPROD) AND (PF.CDFILIAL(+) = :CDFILIAL) AND (PF.CDPRODUTO(+) = P.CDPRODUTO)";
    	$params = array(
            'CDARVPROD' 	=> $prod,
            'CDFILIAL'		=> $row[0]['CDFILIAL']
		);
		$query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        return $result = $query->fetchAll();
    }
    
    public function exportRegras(DTO\Request $request, DTO\Response $response){
    	try{
	    	$rows = $request->getDataSet()->getRows();
	    	$rowsToUpdate = $rows[0]["params"];
	    	$filialOrig = $rows[0]["CDFILIALORIG"];
	    	unset($rows[0]["params"]);
	    	
	    	if($rows[0]["ALLCDFILIALDEST"] != "T"){ 
	    		$filiaisDest = $rows[0]["NMFILIALDEST"];
	    	}else 
	    		$filiaisDest = [$rows[0]["ALLCDFILIALDEST"]];
	    	
	    	//Acrescentar
	    	if($rows[0]["TIPOEXPORT"] == 0){
	    		foreach($rowsToUpdate as $key => &$row){
	    			foreach($filiaisDest as $keyfili => &$filial){
		    			$result = self::verifyIsNew($row, $filial);
		    			if(empty($result)){
		    				self::insertRow($row, $filial, $filialOrig);
		    			}
	    			}
	    		}
	    	}else{
		    	//Sobrepor
		    	foreach($filiaisDest as $keyfili => &$filial){
			    	self::deleteRegra($filial, $filialOrig);
	    		}
	    		foreach($rowsToUpdate as $key => &$row){
	    			foreach($filiaisDest as $keyfili => &$filial){
			    		self::insertRow($row, $filial, $filialOrig);
	    			}
	    		}
	    	}
    	}catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function verifyIsNew($row, $filialDest){
    	$sql = "SELECT CDPRODORIGRE FROM REGRAREVRECE WHERE (CDFILIAL = :CDFILIAL OR (:CDFILIAL = 'T')) AND ((CDPRODORIGRE = :CDPRODORIGRE AND QTPRODORIGRE = :QTPRODORIGRE) OR (CDPRODDESTRE = :CDPRODDESTRE AND QTPRODDESTRE = :QTPRODDESTRE))";
    	
    	$params = array(
            'CDFILIAL' 		=> $filialDest,
            'CDPRODORIGRE'	=> $row['CDPRODORIGRE'],
            'QTPRODORIGRE'	=> $row['QTPRODORIGRE'],
            'CDPRODDESTRE'	=> $row['CDPRODDESTRE'],
            'QTPRODDESTRE'	=> $row['QTPRODDESTRE']
		);
		$query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        return $result = $query->fetchAll();
    }
    
    public function deleteRegra($filial, $filialOrig){
    	$sql = "DELETE FROM REGRAREVRECE WHERE (INSTR(:CDFILIAL, CDFILIAL) <> 0 OR (:CDFILIAL = 'T')) AND CDFILIAL <> :CDFILIORIG";
    	$params = array(
            'CDFILIAL' 		=> $filial,
            'CDFILIORIG' 	=> $filialOrig
		);
		$query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function insertRow($row, $filialDest, $filialOrig){
    	$sql = "INSERT INTO REGRAREVRECE (CDFILIAL, CDPRODORIGRE, QTPRODORIGRE, CDPRODDESTRE, QTPRODDESTRE, IDAPLICAREG) SELECT  CDFILIAL, :CDPRODORIGRE, :QTPRODORIGRE, :CDPRODDESTRE, :QTPRODDESTRE, :IDAPLICAREG FROM FILIAL WHERE (CDFILIAL = :CDFILIAL OR :CDFILIAL = 'T') AND CDFILIAL <> :CDFILIORIG";
    	$params = array(
            'CDFILIAL' 		=> $filialDest,
            'CDFILIORIG' 	=> $filialOrig,
            'CDPRODORIGRE'	=> $row['CDPRODORIGRE'],
            'QTPRODORIGRE'	=> $row['QTPRODORIGRE'],
            'CDPRODDESTRE'	=> $row['CDPRODDESTRE'],
            'QTPRODDESTRE'	=> $row['QTPRODDESTRE'],
            'IDAPLICAREG'	=> $row['IDAPLICAREG']
		);
		$query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function revisaoAutomatica(DTO\Request $request, DTO\Response $response){
    	$row = $request->getRow();
    	$cdfilial = array($row['CDFILIAL']);
    	$nrLista = $this->util->insertLista($cdfilial);
    	
    	$procedureRevisao = new StoredProcedure($this->entityManager->getConnection(), 'INTEGRA_RECEITA_REGRAS');
        
        $procedureRevisao->addParam(new Param('P_NRLISTA',    Param::PARAM_INPUT, $nrLista,   		 Param::PARAM_TYPE_STR));
        $procedureRevisao->addParam(new Param('P_CDPRATOINI', Param::PARAM_INPUT, $row['CDPRATOINI'],  Param::PARAM_TYPE_STR));
        $procedureRevisao->addParam(new Param('P_CDPRATOFIN', Param::PARAM_INPUT, $row['CDPRATOFIM'],  Param::PARAM_TYPE_STR));
        $procedureRevisao->addParam(new Param('P_TIPOIMPORT', Param::PARAM_INPUT, $row['TIPOIMPORT'],  Param::PARAM_TYPE_STR));
        $procedureRevisao->execute();
    }
}